/**
 * A plugin to enable iOS In-App Purchases.
 *
 * Copyright (c) Matt Kane 2011
 * Copyright (c) Guillaume Charhon 2012
 */

var InAppPurchaseManager = function () {
    cordova.exec(null, null, 'InAppPurchaseManager', 'setup', []);
};

/**
 * Makes an in-app purchase.
 *
 * @param {String} productId The product identifier. e.g. "com.example.MyApp.myproduct"
 * @param {int} quantity
 */

InAppPurchaseManager.prototype.makePurchase = function (productId, quantity) {
    var q = parseInt(quantity);
    if (!q) {
        q = 1;
    }
    return cordova.exec(null, null, 'InAppPurchaseManager', 'makePurchase', [productId, q]);
};

/**
 * Asks the payment queue to restore previously completed purchases.
 * The restored transactions are passed to the onRestored callback, so make sure you define a handler for that first.
 *
 */

InAppPurchaseManager.prototype.restoreCompletedTransactions = function () {
    var samlagetProductIDs = [
        "no.samlaget.fibi",
        "no.samlaget.ccponny",
        "no.samlaget.kroketemanen",
        "no.samlaget.draken",
        "no.samlaget.katten",
        "no.samlaget.feline",
        "no.samlaget.superheltane"
    ];
    if (AppUtils.DeviceType.iOS) {
        return cordova.exec(null, null, 'InAppPurchaseManager', 'restoreCompletedTransactions', []);
    } else if (AppUtils.DeviceType.ANDROID) {
        return cordova.exec(null, null, 'InAppPurchaseManager', 'restoreCompletedTransactions', [samlagetProductIDs]);
    }
    return console.log("Restore operation not supported on your platform");
};


/**
 * Retrieves the localised product data, including price (as a localised string), name, description.
 * You must call this before attempting to make a purchase.
 *
 * @param {String} productId The product identifier. e.g. "com.example.MyApp.myproduct"
 * @param {Function} successCallback Called once for each returned product id. Signature is function(productId, title, description, price)
 * @param {Function} failCallback Called once for each invalid product id. Signature is function(productId)
 */

InAppPurchaseManager.prototype.requestProductData = function (productId, successCallback, failCallback) {
    var key = 'f' + this.callbackIdx++;
    window.plugins.inAppPurchaseManager.callbackMap[key] = {
        success: function (productId, title, description, price) {
            if (productId == '__DONE') {
                delete window.plugins.inAppPurchaseManager.callbackMap[key];
                return;
            }
            successCallback(productId, title, description, price);
        },
        fail: failCallback
    };

    var callback = 'window.plugins.inAppPurchaseManager.callbackMap.' + key;
    cordova.exec(function () {
            console.log("Request product data OK ... waiting for response");
        }, function (error) {
            console.log("Request product data JS error: " + error);
        }, 'InAppPurchaseManager', 'requestProductData',
        [productId, callback + ".success", callback + ".fail"]);
};

/**
 * Retrieves localised product data, including price (as localised
 * string), name, description of multiple products.
 *
 * @param {Array} productIds
 *   An array of product identifier strings.
 *
 * @param {Function} callback
 *   Called once with the result of the products request. Signature:
 *
 *     function(validProducts, invalidProductIds)
 *
 *   where validProducts receives an array of objects of the form
 *
 *     {
 *      id: "<productId>",
 *      title: "<localised title>",
 *      description: "<localised description>",
 *      price: "<localised price>"
 *     }
 *
 *  and invalidProductIds receives an array of product identifier
 *  strings which were rejected by the app store.
 */
InAppPurchaseManager.prototype.requestProductsData = function (productIds, successCallback) {
    var key = 'b' + this.callbackIdx++;
    window.plugins.inAppPurchaseManager.callbackMap[key] = {
        success: function (validProducts, invalidProductIds) {
            successCallback(validProducts, invalidProductIds);
            delete window.plugins.inAppPurchaseManager.callbackMap[key];
        }
    };
    var callbackName = 'window.plugins.inAppPurchaseManager.callbackMap.' + key;
    cordova.exec(function () {
            console.log("Request product prices OK ... waiting for response");
        }, function (error) {
            console.log("Request product prices JS error: " + error);
        }, 'InAppPurchaseManager', 'requestProductsData',
        [productIds, callbackName + ".success"]);
};

/* function(transactionIdentifier, productId, transactionReceipt) */
InAppPurchaseManager.prototype.onPurchased = null;  // this will be overridden in iap_utils.js

/* function(originalTransactionIdentifier, productId, originalTransactionReceipt) */
InAppPurchaseManager.prototype.onRestored = null;   // this will be overridden in iap_utils.js

/* function(errorCode, errorText) */
InAppPurchaseManager.prototype.onFailed = null; // this will be overridden in iap_utils.js

/* function() */
InAppPurchaseManager.prototype.onRestoreCompletedTransactionsFinished = null;

/* function(errorCode) */
InAppPurchaseManager.prototype.onRestoreCompletedTransactionsFailed = null;

/* This is called from native.*/

InAppPurchaseManager.prototype.updatedTransactionCallback = function (state, errorCode, errorText, productId) {
    console.log("JS updatedTransactionCallback, new state is: " + state);

    switch (state) {
        case "PaymentTransactionStatePurchased":
            if (window.plugins.inAppPurchaseManager.onPurchased)
                window.plugins.inAppPurchaseManager.onPurchased(productId);
            return;

        case "PaymentTransactionStateFailed":
            if (window.plugins.inAppPurchaseManager.onFailed)
                window.plugins.inAppPurchaseManager.onFailed(errorCode, errorText);
            return;

        case "PaymentTransactionStateRestored":
            if (window.plugins.inAppPurchaseManager.onRestored)
                window.plugins.inAppPurchaseManager.onRestored(productId);
            return;
    }
};

/*
 * This queue stuff is here because we may be sent events before listeners have been registered. This is because if we have 
 * incomplete transactions when we quit, the app will try to run these when we resume. If we don't register to receive these
 * right away then they may be missed. As soon as a callback has been registered then it will be sent any events waiting
 * in the queue.
 */

InAppPurchaseManager.prototype.runQueue = function () {
    if (!this.eventQueue.length || (!this.onPurchased && !this.onFailed && !this.onRestored)) {
        return;
    }
    var args;
    /* We can't work directly on the queue, because we're pushing new elements onto it */
    var queue = this.eventQueue.slice();
    this.eventQueue = [];
    while (args = queue.shift()) {
        this.updatedTransactionCallback.apply(this, args);
    }
    if (!this.eventQueue.length) {
        this.unWatchQueue();
    }
};

InAppPurchaseManager.prototype.watchQueue = function () {
    if (this.timer) {
        return;
    }
    this.timer = setInterval("window.plugins.inAppPurchaseManager.runQueue()", 10000);
};

InAppPurchaseManager.prototype.unWatchQueue = function () {
    if (this.timer) {
        clearInterval(this.timer);
        this.timer = null;
    }
};

// Android Section

InAppPurchaseManager.prototype.startAndroidIABForProduct = function (productSKU, successCallback, failureCallback) {
    cordova.exec(successCallback, failureCallback, "InAppPurchaseManager", "startProductPurchase", [productSKU]);
};

InAppPurchaseManager.prototype.startAndroidIABProductsQuery = function (iapProductIds, restore, updatePrices, successCallback, failureCallback) {
    cordova.exec(successCallback, failureCallback, "InAppPurchaseManager", "queryProductsInfo", [iapProductIds, restore, updatePrices]);
};

// END Android


InAppPurchaseManager.prototype.callbackMap = {};
InAppPurchaseManager.prototype.callbackIdx = 0;
InAppPurchaseManager.prototype.eventQueue = [];
InAppPurchaseManager.prototype.timer = null;

InAppPurchaseManager.install = function () {
    console.log("InAppPurchaseManager install");
    if (!window.plugins) {
        window.plugins = {};
    }
    window.plugins.inAppPurchaseManager = new InAppPurchaseManager();
    return window.plugins.inAppPurchaseManager;
};

if (typeof (cordova) != "undefined") {
    console.log("Will run cordova.addConstructor");
    cordova.addConstructor(InAppPurchaseManager.install);
}